// Example project for plugin Journeyman's Minimap

#include "MinimapExample.h"
#include "MinimapExampleCharacter.h"

#include "MapEnums.h"
#include "MapIconComponent.h"
#include "MapViewComponent.h"
#include "MapRevealerComponent.h"

AMinimapExampleCharacter::AMinimapExampleCharacter()
{
	// Set size for collision capsule
	GetCapsuleComponent()->InitCapsuleSize(42.f, 96.0f);
	
	// set our turn rates for input
	BaseTurnRate = 45.f;
	BaseLookUpRate = 45.f;
	
	// Don't rotate when the controller rotates. Let that just affect the camera.
	bUseControllerRotationPitch = false;
	bUseControllerRotationYaw = false;
	bUseControllerRotationRoll = false;
	
	// Configure character movement
	GetCharacterMovement()->bOrientRotationToMovement = true;
	GetCharacterMovement()->RotationRate = FRotator(0.0f, 540.0f, 0.0f);
	GetCharacterMovement()->JumpZVelocity = 600.f;
	GetCharacterMovement()->AirControl = 0.2f;

	// Create a camera boom (pulls in towards the player if there is a collision)
	CameraBoom = CreateDefaultSubobject<USpringArmComponent>(TEXT("CameraBoom"));
	CameraBoom->SetupAttachment(RootComponent);
	CameraBoom->bUsePawnControlRotation = true;
	CameraBoom->TargetArmLength = 800.0f;

	// Create a follow camera
	FollowCamera = CreateDefaultSubobject<UCameraComponent>(TEXT("FollowCamera"));
	FollowCamera->SetupAttachment(CameraBoom, USpringArmComponent::SocketName);
	FollowCamera->bUsePawnControlRotation = false;

	// The following code is character setup for Journeyman's Minimap

	// MapIconComponent makes the character appear on the minimap
	static ConstructorHelpers::FObjectFinder<UTexture2D> PlayerIcon(TEXT("/MinimapPlugin/Textures/Icons/T_Icon_Placeholder"));
	MapIcon = CreateDefaultSubobject<UMapIconComponent>(TEXT("MapIcon"));
	MapIcon->SetupAttachment(GetRootComponent());
	// Set the player icon as texture
	MapIcon->SetIconTexture(PlayerIcon.Object);
	// The icon will rotate to represent the character's rotation
	MapIcon->SetIconRotates(true);

	// MapViewComponent allows the minimap to follow the character
	MapView = CreateDefaultSubobject<UMapViewComponent>(TEXT("MapView"));
	MapView->SetupAttachment(GetRootComponent());
	// How far the player can see on the minimap. Change during gameplay to apply zooming. Aspect ratio must match the minimap widget's aspect ratio.
	MapView->SetViewExtent(1024.f, 1024.f);
	// Whether the minimap should rotate with the player. Set to false for fixed rotation minimap.
	MapView->RotationMode = EMapViewRotationMode::InheritYaw;
	// The angle that represents north
	MapView->InheritedYawOffset = 90.0f;

	// MapRevealerComponent makes the character reveal fog
	MapRevealer = CreateDefaultSubobject<UMapRevealerComponent>(TEXT("MapRevealer"));
	MapRevealer->SetupAttachment(GetRootComponent());
	// Sets the radius that the player reveals 100%, and the distance over which it linearly drops off
	MapRevealer->SetRevealExtent(200.f, 200.f);
	MapRevealer->SetRevealDropOffDistance(100.f);
	// Reveal temporarily to make areas hidden again after leaving. Reveal permanently to make areas explored forever.
	MapRevealer->SetRevealMode(EMapFogRevealMode::Temporary);
}

//////////////////////////////////////////////////////////////////////////
// Input

void AMinimapExampleCharacter::SetupPlayerInputComponent(class UInputComponent* PlayerInputComponent)
{
	// Third person controls
	check(PlayerInputComponent);
	PlayerInputComponent->BindAction("Jump", IE_Pressed, this, &ACharacter::Jump);
	PlayerInputComponent->BindAction("Jump", IE_Released, this, &ACharacter::StopJumping);
	PlayerInputComponent->BindAxis("MoveForward", this, &AMinimapExampleCharacter::MoveForward);
	PlayerInputComponent->BindAxis("MoveRight", this, &AMinimapExampleCharacter::MoveRight);
	PlayerInputComponent->BindAxis("Turn", this, &APawn::AddControllerYawInput);
	PlayerInputComponent->BindAxis("TurnRate", this, &AMinimapExampleCharacter::TurnAtRate);
	PlayerInputComponent->BindAxis("LookUp", this, &APawn::AddControllerPitchInput);
	PlayerInputComponent->BindAxis("LookUpRate", this, &AMinimapExampleCharacter::LookUpAtRate);
}

void AMinimapExampleCharacter::TurnAtRate(float Rate)
{
	AddControllerYawInput(Rate * BaseTurnRate * GetWorld()->GetDeltaSeconds());
}

void AMinimapExampleCharacter::LookUpAtRate(float Rate)
{
	AddControllerPitchInput(Rate * BaseLookUpRate * GetWorld()->GetDeltaSeconds());
}

void AMinimapExampleCharacter::MoveForward(float Value)
{
	if ((Controller != NULL) && (Value != 0.0f))
	{
		const FRotator Rotation = Controller->GetControlRotation();
		const FRotator YawRotation(0, Rotation.Yaw, 0);
		const FVector Direction = FRotationMatrix(YawRotation).GetUnitAxis(EAxis::X);
		AddMovementInput(Direction, Value);
	}
}

void AMinimapExampleCharacter::MoveRight(float Value)
{
	if ( (Controller != NULL) && (Value != 0.0f) )
	{
		const FRotator Rotation = Controller->GetControlRotation();
		const FRotator YawRotation(0, Rotation.Yaw, 0);
		const FVector Direction = FRotationMatrix(YawRotation).GetUnitAxis(EAxis::Y);
		AddMovementInput(Direction, Value);
	}
}
